@@ -20,7 +20,6 @@
  */
 
 #include <stdlib.h>
-#include <ctype.h>
 #include <math.h>
 #include <string.h>
 
@@ -707,11 +706,14 @@ static FieldMap variant_map[] = {
 };
 
 static FieldMap weight_map[] = {
-  { 300, "Light" },
+  { PANGO_WEIGHT_ULTRALIGHT, "Ultra-Light" },
+  { PANGO_WEIGHT_LIGHT, "Light" },
   { PANGO_WEIGHT_NORMAL, NULL },
   { 500, "Medium" },
   { 600, "Semi-Bold" },
-  { PANGO_WEIGHT_BOLD, "Bold" }
+  { PANGO_WEIGHT_BOLD, "Bold" },
+  { PANGO_WEIGHT_ULTRABOLD, "Ultra-Bold" },
+  { PANGO_WEIGHT_HEAVY, "Heavy" }
 };
 
 static FieldMap stretch_map[] = {
@@ -763,11 +765,11 @@ getword (const char *str, const char *last, size_t *wordlen)
 {
   const char *result;
   
-  while (last > str && isspace (*(last - 1)))
+  while (last > str && g_ascii_isspace (*(last - 1)))
     last--;
 
   result = last;
-  while (result > str && !isspace (*(result - 1)))
+  while (result > str && !g_ascii_isspace (*(result - 1)))
     result--;
 
   *wordlen = last - result;
@@ -791,8 +793,7 @@ getword (const char *str, const char *last, size_t *wordlen)
  * default values. If SIZE is missing, the size in the resulting font
  * description will be set to 0.
  * 
- * Return value: a new #PangoFontDescription, or %NULL if the 
- * string could not be parsed. 
+ * Return value: a new #PangoFontDescription. 
  **/
 PangoFontDescription *
 pango_font_description_from_string (const char *str)
@@ -855,16 +856,16 @@ pango_font_description_from_string (const char *str)
   /* Remainder (str => p) is family list. Trim off trailing commas and leading and trailing white space
    */
 
-  while (last > str && isspace (*(last - 1)))
+  while (last > str && g_ascii_isspace (*(last - 1)))
     last--;
 
   if (last > str && *(last - 1) == ',')
     last--;
 
-  while (last > str && isspace (*(last - 1)))
+  while (last > str && g_ascii_isspace (*(last - 1)))
     last--;
 
-  while (isspace (*str))
+  while (g_ascii_isspace (*str))
     str++;
 
   if (str != last)
