@@ -40,6 +40,8 @@ struct _PangoXftFont
   PangoFontDescription *description;
   PangoOTInfo *ot_info;
 
+  GSList *metrics_by_lang;
+
   guint16 mini_width;
   guint16 mini_height;
   guint16 mini_pad; 
@@ -64,7 +66,8 @@ void           _pango_xft_font_map_get_info     (PangoFontMap                *fo
 						 Display                    **display,
 						 int                         *screen);
 
-PangoFontDescription * _pango_xft_font_desc_from_pattern (XftPattern *pattern);
+PangoFontDescription * _pango_xft_font_desc_from_pattern (XftPattern *pattern,
+							  gboolean    include_size);
 
 G_END_DECLS
 
