@@ -42,6 +42,48 @@
 #endif
 #define SOEXT_LEN (strlen (SOEXT))
 
+static gboolean
+string_needs_escape (const char *str)
+{
+  while (TRUE)
+    {
+      char c = *str++;
+
+      if (!c)
+	return FALSE;
+      else if (c == '\"' || g_ascii_isspace (c))
+	return TRUE;
+    }
+}
+
+static char *
+escape_string (const char *str)
+{
+  GString *result = g_string_new ("");
+
+  while (TRUE)
+    {
+      char c = *str++;
+      
+      switch (c)
+	{
+	case '\0':
+	  goto done;
+	case '\n':
+	  g_string_append (result, "\\n");
+	  break;
+	case '\"':
+	  g_string_append (result, "\\\"");
+	  break;
+	default:
+	  g_string_append_c (result, c);
+	}
+    }
+
+ done:
+  return g_string_free (result, FALSE);
+}
+
 void 
 query_module (const char *dir, const char *name)
 {
@@ -78,10 +120,10 @@ query_module (const char *dir, const char *name)
 	  const gchar *quote;
 	  gchar *quoted_path;
 
-	  if (strchr (path, ' ') != NULL)
+	  if (string_needs_escape (path))
 	    {
 	      quote = "\"";
-	      quoted_path = g_strescape (path, NULL);
+	      quoted_path = escape_string (path);
 	    }
 	  else
 	    {
@@ -89,27 +131,27 @@ query_module (const char *dir, const char *name)
 	      quoted_path = g_strdup (path);
 	    }
 	  
-	  g_print ("%s%s%s %s %s %s ", quote, quoted_path, quote,
-		   engines[i].id, engines[i].engine_type, engines[i].render_type);
+	  printf ("%s%s%s %s %s %s ", quote, quoted_path, quote,
+		  engines[i].id, engines[i].engine_type, engines[i].render_type);
 	  g_free (quoted_path);
 
 	  for (j=0; j < engines[i].n_ranges; j++)
 	    {
 	      if (j != 0)
-		g_print (" ");
-	      g_print ("%d-%d:%s",
-		       engines[i].ranges[j].start,
-		       engines[i].ranges[j].end,
-		       engines[i].ranges[j].langs);
+		printf (" ");
+	      printf ("%d-%d:%s",
+		      engines[i].ranges[j].start,
+		      engines[i].ranges[j].end,
+		      engines[i].ranges[j].langs);
 	    }
-	  g_print ("\n");
-	  }
+	  printf ("\n");
+	}
     }
   else
     {
       fprintf (stderr, "%s does not export Pango module API\n", path);
     }
-
+  
   g_free (path);
   if (module)
     g_module_close (module);
